using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading;
using System.Reflection;

namespace ImageResizer
{
	/// <summary>
	/// Summary description for ImageResizerForm.
	/// </summary>
	public class ImageResizerForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem FileMenu;
		private System.Windows.Forms.MenuItem FileSetInputDir;
		private System.Windows.Forms.MenuItem FileSetOutputDir;
		private System.Windows.Forms.MenuItem FileMenuSeparator1;
		private System.Windows.Forms.MenuItem FileExit;
		private System.Windows.Forms.MenuItem EditMenu;
		private System.Windows.Forms.MenuItem EditSelectAll;
		private System.Windows.Forms.MenuItem EditSelectNone;
		private System.Windows.Forms.CheckedListBox InputFilesListBox;
		private System.Windows.Forms.ProgressBar ResizeProgressBar;
		private System.Windows.Forms.GroupBox ResizeSettingsGroupBox;
		private System.Windows.Forms.RadioButton ResizeMaxDimension;
		private System.Windows.Forms.NumericUpDown MaximumDimensionEntry;
		private System.Windows.Forms.RadioButton ResizeDimensionPercentage;
		private System.Windows.Forms.NumericUpDown DimensionPercentageEntry;
		private System.Windows.Forms.RadioButton ResizeAreaPercentage;
		private System.Windows.Forms.NumericUpDown AreaPercentageEntry;
		private System.Windows.Forms.RadioButton ResizeExactSize;
		private System.Windows.Forms.Label WidthLabel;
		private System.Windows.Forms.NumericUpDown ExactWidthEntry;
		private System.Windows.Forms.Label HeightLabel;
		private System.Windows.Forms.NumericUpDown ExactHeightEntry;
		private System.Windows.Forms.CheckBox OverwriteCheckBox;
		private System.Windows.Forms.Label InputDirectoryLabel;
		private System.Windows.Forms.Label OutputDirectoryLabel;
		private System.Windows.Forms.Label CurrentFilenameLabel;
		private System.Windows.Forms.PictureBox CurrentPictureBox;
		private System.Windows.Forms.Button ResizeButton;
		private System.Windows.Forms.FolderBrowserDialog folderBrowserDialog;
		private System.Windows.Forms.ToolTip ResizeToolTips;
		private System.ComponentModel.IContainer components;

		//
		// Implementation variables:
		//
		string InputDir, OutputDir;
		private System.Windows.Forms.MenuItem EditShowUnprocessedFiles;
		private System.Windows.Forms.MenuItem EditShowAllFiles;
		string ResizeMode = "DimensionPercentage";

		// Because we want to update the UI with actual data--a string--we need to create
		// our own EventArgs-derived class.
		public class ResizeFilenameArgs : EventArgs 
		{
			public string filename;
			public ResizeFilenameArgs(string s) 
			{
				filename = s;
			}
		}

		// Declare a delegate class to hold the prototype information we want to enforce
		// for methods which subscribe to the event we'll define.  Because this event is
		// an ordinary WinForms type event, it has to take a source object and an
		// EventArgs-derived structure.
		public delegate void ResizeFilenameHandler(object worker, ResizeFilenameArgs e);
		// And here we use the delegate--the prototype information--to declare an actual
		// event object.  OnWorkerOutput is a field in Worker, which subscribers can
		// compose (+=) their event handlers onto.
		public event ResizeFilenameHandler OnFilenameChangeEvent;

		// another one for the progress event:
		public delegate void ResizeProgressHandler(object worker, EventArgs e); // uses generic event args
		public event ResizeProgressHandler OnProgressEvent;

		// another one for the done event:
		public delegate void ResizeDoneHandler(object worker, EventArgs e); // uses generic event args
		public event ResizeDoneHandler OnResizeDoneEvent;

		public ImageResizerForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			// initialize these, or else "...not set to instance of an object" errors occur.
			InputDir = OutputDir = "";
			// subscribe to the worker thread events:
			OnFilenameChangeEvent += new ResizeFilenameHandler(OnFilename);
			OnProgressEvent       += new ResizeProgressHandler(OnProgress);
			OnResizeDoneEvent     += new ResizeDoneHandler(OnResizeDone);

			// set tooltips:
			ResizeToolTips.SetToolTip(InputFilesListBox,
				"Use File->Set Input Directory to set the list of files to work with.\n" +
				"You can then select or unselect files to work with.");
			ResizeToolTips.SetToolTip(ResizeMaxDimension,
				"Resize images so that their longest dimension is this many pixels.\n" + 
				"The resized images will have the same proportions as the originals.");
			ResizeToolTips.SetToolTip(ResizeDimensionPercentage,
				"Resize length and width of images to this percentage of their original length and width.\n" + 
				"The resized images will have the same proportions as the originals.");
			ResizeToolTips.SetToolTip(ResizeAreaPercentage,
				"Resize the image to this percentage of its original area.\n" + 
				"The resized images will have the same proportions as the originals.");
			ResizeToolTips.SetToolTip(ResizeExactSize,
				"Resize images to this specific width and height.\n" + 
				"The resized images may not have the same proportions as the originals.");
			ResizeToolTips.SetToolTip(OverwriteCheckBox,
				"Use this option to control whether to overwrite an existing file with an output file,\n" +
				"if a file with the same name and location as the output file already exists.");
			ResizeToolTips.SetToolTip(InputDirectoryLabel,
				"Displays the directory where the files you are working with are stored.");
			ResizeToolTips.SetToolTip(OutputDirectoryLabel,
				"Displays the directory where resized image files will be placed.\n" +
				"The default is to make a directory called \"Resized\" in the input directory.");
			ResizeToolTips.SetToolTip(CurrentFilenameLabel,
				"Displays the name of the current image file being processed");
			ResizeToolTips.SetToolTip(CurrentPictureBox,
				"This area shows the current image being processed."); 
			ResizeToolTips.SetToolTip(ResizeButton,
				"Click this button to begin the resizing operation after selecting the files you want" +
				"to resize and selecting your resize settings.\n");
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ImageResizerForm));
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.FileMenu = new System.Windows.Forms.MenuItem();
			this.FileSetInputDir = new System.Windows.Forms.MenuItem();
			this.FileSetOutputDir = new System.Windows.Forms.MenuItem();
			this.FileMenuSeparator1 = new System.Windows.Forms.MenuItem();
			this.FileExit = new System.Windows.Forms.MenuItem();
			this.EditMenu = new System.Windows.Forms.MenuItem();
			this.EditSelectAll = new System.Windows.Forms.MenuItem();
			this.EditSelectNone = new System.Windows.Forms.MenuItem();
			this.InputFilesListBox = new System.Windows.Forms.CheckedListBox();
			this.ResizeProgressBar = new System.Windows.Forms.ProgressBar();
			this.ResizeSettingsGroupBox = new System.Windows.Forms.GroupBox();
			this.CurrentPictureBox = new System.Windows.Forms.PictureBox();
			this.OverwriteCheckBox = new System.Windows.Forms.CheckBox();
			this.OutputDirectoryLabel = new System.Windows.Forms.Label();
			this.InputDirectoryLabel = new System.Windows.Forms.Label();
			this.ResizeButton = new System.Windows.Forms.Button();
			this.ExactHeightEntry = new System.Windows.Forms.NumericUpDown();
			this.HeightLabel = new System.Windows.Forms.Label();
			this.WidthLabel = new System.Windows.Forms.Label();
			this.ExactWidthEntry = new System.Windows.Forms.NumericUpDown();
			this.AreaPercentageEntry = new System.Windows.Forms.NumericUpDown();
			this.DimensionPercentageEntry = new System.Windows.Forms.NumericUpDown();
			this.MaximumDimensionEntry = new System.Windows.Forms.NumericUpDown();
			this.ResizeExactSize = new System.Windows.Forms.RadioButton();
			this.ResizeAreaPercentage = new System.Windows.Forms.RadioButton();
			this.ResizeDimensionPercentage = new System.Windows.Forms.RadioButton();
			this.ResizeMaxDimension = new System.Windows.Forms.RadioButton();
			this.CurrentFilenameLabel = new System.Windows.Forms.Label();
			this.folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();
			this.ResizeToolTips = new System.Windows.Forms.ToolTip(this.components);
			this.EditShowUnprocessedFiles = new System.Windows.Forms.MenuItem();
			this.EditShowAllFiles = new System.Windows.Forms.MenuItem();
			this.ResizeSettingsGroupBox.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.ExactHeightEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.ExactWidthEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.AreaPercentageEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.DimensionPercentageEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.MaximumDimensionEntry)).BeginInit();
			this.SuspendLayout();
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.FileMenu,
																					 this.EditMenu});
			// 
			// FileMenu
			// 
			this.FileMenu.Index = 0;
			this.FileMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.FileSetInputDir,
																					 this.FileSetOutputDir,
																					 this.FileMenuSeparator1,
																					 this.FileExit});
			this.FileMenu.Text = "&File";
			// 
			// FileSetInputDir
			// 
			this.FileSetInputDir.Index = 0;
			this.FileSetInputDir.Text = "Set &Input Directory...";
			this.FileSetInputDir.Click += new System.EventHandler(this.FileSetInputDir_Click);
			// 
			// FileSetOutputDir
			// 
			this.FileSetOutputDir.Index = 1;
			this.FileSetOutputDir.Text = "Set &Output Directory...";
			this.FileSetOutputDir.Click += new System.EventHandler(this.FileSetOutputDir_Click);
			// 
			// FileMenuSeparator1
			// 
			this.FileMenuSeparator1.Index = 2;
			this.FileMenuSeparator1.Text = "-";
			// 
			// FileExit
			// 
			this.FileExit.Index = 3;
			this.FileExit.Text = "E&xit";
			this.FileExit.Click += new System.EventHandler(this.FileExit_Click);
			// 
			// EditMenu
			// 
			this.EditMenu.Index = 1;
			this.EditMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.EditSelectAll,
																					 this.EditSelectNone,
																					 this.EditShowUnprocessedFiles,
																					 this.EditShowAllFiles});
			this.EditMenu.Text = "&Edit";
			// 
			// EditSelectAll
			// 
			this.EditSelectAll.Index = 0;
			this.EditSelectAll.Text = "Select A&ll";
			this.EditSelectAll.Click += new System.EventHandler(this.EditSelectAll_Click);
			// 
			// EditSelectNone
			// 
			this.EditSelectNone.Index = 1;
			this.EditSelectNone.Text = "Select &None";
			this.EditSelectNone.Click += new System.EventHandler(this.EditSelectNone_Click);
			// 
			// InputFilesListBox
			// 
			this.InputFilesListBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.InputFilesListBox.CheckOnClick = true;
			this.InputFilesListBox.IntegralHeight = false;
			this.InputFilesListBox.Location = new System.Drawing.Point(0, 0);
			this.InputFilesListBox.MultiColumn = true;
			this.InputFilesListBox.Name = "InputFilesListBox";
			this.InputFilesListBox.Size = new System.Drawing.Size(704, 680);
			this.InputFilesListBox.Sorted = true;
			this.InputFilesListBox.TabIndex = 0;
			this.InputFilesListBox.UseTabStops = false;
			// 
			// ResizeProgressBar
			// 
			this.ResizeProgressBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.ResizeProgressBar.Location = new System.Drawing.Point(0, 680);
			this.ResizeProgressBar.Name = "ResizeProgressBar";
			this.ResizeProgressBar.Size = new System.Drawing.Size(704, 21);
			this.ResizeProgressBar.TabIndex = 1;
			// 
			// ResizeSettingsGroupBox
			// 
			this.ResizeSettingsGroupBox.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.ResizeSettingsGroupBox.Controls.Add(this.CurrentPictureBox);
			this.ResizeSettingsGroupBox.Controls.Add(this.OverwriteCheckBox);
			this.ResizeSettingsGroupBox.Controls.Add(this.OutputDirectoryLabel);
			this.ResizeSettingsGroupBox.Controls.Add(this.InputDirectoryLabel);
			this.ResizeSettingsGroupBox.Controls.Add(this.ResizeButton);
			this.ResizeSettingsGroupBox.Controls.Add(this.ExactHeightEntry);
			this.ResizeSettingsGroupBox.Controls.Add(this.HeightLabel);
			this.ResizeSettingsGroupBox.Controls.Add(this.WidthLabel);
			this.ResizeSettingsGroupBox.Controls.Add(this.ExactWidthEntry);
			this.ResizeSettingsGroupBox.Controls.Add(this.AreaPercentageEntry);
			this.ResizeSettingsGroupBox.Controls.Add(this.DimensionPercentageEntry);
			this.ResizeSettingsGroupBox.Controls.Add(this.MaximumDimensionEntry);
			this.ResizeSettingsGroupBox.Controls.Add(this.ResizeExactSize);
			this.ResizeSettingsGroupBox.Controls.Add(this.ResizeAreaPercentage);
			this.ResizeSettingsGroupBox.Controls.Add(this.ResizeDimensionPercentage);
			this.ResizeSettingsGroupBox.Controls.Add(this.ResizeMaxDimension);
			this.ResizeSettingsGroupBox.Controls.Add(this.CurrentFilenameLabel);
			this.ResizeSettingsGroupBox.Location = new System.Drawing.Point(704, 0);
			this.ResizeSettingsGroupBox.Name = "ResizeSettingsGroupBox";
			this.ResizeSettingsGroupBox.Size = new System.Drawing.Size(264, 702);
			this.ResizeSettingsGroupBox.TabIndex = 2;
			this.ResizeSettingsGroupBox.TabStop = false;
			this.ResizeSettingsGroupBox.Text = "Resize Settings";
			// 
			// CurrentPictureBox
			// 
			this.CurrentPictureBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.CurrentPictureBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.CurrentPictureBox.Location = new System.Drawing.Point(8, 392);
			this.CurrentPictureBox.Name = "CurrentPictureBox";
			this.CurrentPictureBox.Size = new System.Drawing.Size(248, 272);
			this.CurrentPictureBox.TabIndex = 15;
			this.CurrentPictureBox.TabStop = false;
			// 
			// OverwriteCheckBox
			// 
			this.OverwriteCheckBox.Location = new System.Drawing.Point(8, 240);
			this.OverwriteCheckBox.Name = "OverwriteCheckBox";
			this.OverwriteCheckBox.Size = new System.Drawing.Size(248, 24);
			this.OverwriteCheckBox.TabIndex = 14;
			this.OverwriteCheckBox.Text = "Overwrite if output file already exists?";
			// 
			// OutputDirectoryLabel
			// 
			this.OutputDirectoryLabel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.OutputDirectoryLabel.Location = new System.Drawing.Point(8, 312);
			this.OutputDirectoryLabel.Name = "OutputDirectoryLabel";
			this.OutputDirectoryLabel.Size = new System.Drawing.Size(248, 32);
			this.OutputDirectoryLabel.TabIndex = 13;
			this.OutputDirectoryLabel.Text = "Output Directory: <none>";
			// 
			// InputDirectoryLabel
			// 
			this.InputDirectoryLabel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.InputDirectoryLabel.Location = new System.Drawing.Point(8, 272);
			this.InputDirectoryLabel.Name = "InputDirectoryLabel";
			this.InputDirectoryLabel.Size = new System.Drawing.Size(248, 32);
			this.InputDirectoryLabel.TabIndex = 12;
			this.InputDirectoryLabel.Text = "Input Directory: <none>";
			// 
			// ResizeButton
			// 
			this.ResizeButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.ResizeButton.Enabled = false;
			this.ResizeButton.Location = new System.Drawing.Point(128, 670);
			this.ResizeButton.Name = "ResizeButton";
			this.ResizeButton.Size = new System.Drawing.Size(128, 23);
			this.ResizeButton.TabIndex = 11;
			this.ResizeButton.Text = "Resize Selected Files";
			this.ResizeButton.Click += new System.EventHandler(this.ResizeButton_Click);
			// 
			// ExactHeightEntry
			// 
			this.ExactHeightEntry.Enabled = false;
			this.ExactHeightEntry.Location = new System.Drawing.Point(184, 208);
			this.ExactHeightEntry.Maximum = new System.Decimal(new int[] {
																			 100000,
																			 0,
																			 0,
																			 0});
			this.ExactHeightEntry.Name = "ExactHeightEntry";
			this.ExactHeightEntry.Size = new System.Drawing.Size(72, 20);
			this.ExactHeightEntry.TabIndex = 10;
			this.ExactHeightEntry.Value = new System.Decimal(new int[] {
																		   480,
																		   0,
																		   0,
																		   0});
			// 
			// HeightLabel
			// 
			this.HeightLabel.Location = new System.Drawing.Point(144, 208);
			this.HeightLabel.Name = "HeightLabel";
			this.HeightLabel.Size = new System.Drawing.Size(40, 20);
			this.HeightLabel.TabIndex = 9;
			this.HeightLabel.Text = "Height:";
			this.HeightLabel.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// WidthLabel
			// 
			this.WidthLabel.Location = new System.Drawing.Point(24, 208);
			this.WidthLabel.Name = "WidthLabel";
			this.WidthLabel.Size = new System.Drawing.Size(40, 20);
			this.WidthLabel.TabIndex = 8;
			this.WidthLabel.Text = "Width:";
			this.WidthLabel.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// ExactWidthEntry
			// 
			this.ExactWidthEntry.Enabled = false;
			this.ExactWidthEntry.Location = new System.Drawing.Point(64, 208);
			this.ExactWidthEntry.Maximum = new System.Decimal(new int[] {
																			100000,
																			0,
																			0,
																			0});
			this.ExactWidthEntry.Name = "ExactWidthEntry";
			this.ExactWidthEntry.Size = new System.Drawing.Size(72, 20);
			this.ExactWidthEntry.TabIndex = 7;
			this.ExactWidthEntry.Value = new System.Decimal(new int[] {
																		  640,
																		  0,
																		  0,
																		  0});
			// 
			// AreaPercentageEntry
			// 
			this.AreaPercentageEntry.Enabled = false;
			this.AreaPercentageEntry.Location = new System.Drawing.Point(24, 152);
			this.AreaPercentageEntry.Maximum = new System.Decimal(new int[] {
																				200,
																				0,
																				0,
																				0});
			this.AreaPercentageEntry.Minimum = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				0});
			this.AreaPercentageEntry.Name = "AreaPercentageEntry";
			this.AreaPercentageEntry.Size = new System.Drawing.Size(72, 20);
			this.AreaPercentageEntry.TabIndex = 6;
			this.AreaPercentageEntry.Value = new System.Decimal(new int[] {
																			  10,
																			  0,
																			  0,
																			  0});
			// 
			// DimensionPercentageEntry
			// 
			this.DimensionPercentageEntry.Location = new System.Drawing.Point(24, 96);
			this.DimensionPercentageEntry.Maximum = new System.Decimal(new int[] {
																					 200,
																					 0,
																					 0,
																					 0});
			this.DimensionPercentageEntry.Minimum = new System.Decimal(new int[] {
																					 1,
																					 0,
																					 0,
																					 0});
			this.DimensionPercentageEntry.Name = "DimensionPercentageEntry";
			this.DimensionPercentageEntry.Size = new System.Drawing.Size(72, 20);
			this.DimensionPercentageEntry.TabIndex = 5;
			this.DimensionPercentageEntry.Value = new System.Decimal(new int[] {
																				   10,
																				   0,
																				   0,
																				   0});
			// 
			// MaximumDimensionEntry
			// 
			this.MaximumDimensionEntry.Enabled = false;
			this.MaximumDimensionEntry.Location = new System.Drawing.Point(24, 40);
			this.MaximumDimensionEntry.Maximum = new System.Decimal(new int[] {
																				  16384,
																				  0,
																				  0,
																				  0});
			this.MaximumDimensionEntry.Minimum = new System.Decimal(new int[] {
																				  1,
																				  0,
																				  0,
																				  0});
			this.MaximumDimensionEntry.Name = "MaximumDimensionEntry";
			this.MaximumDimensionEntry.Size = new System.Drawing.Size(72, 20);
			this.MaximumDimensionEntry.TabIndex = 4;
			this.MaximumDimensionEntry.Value = new System.Decimal(new int[] {
																				320,
																				0,
																				0,
																				0});
			// 
			// ResizeExactSize
			// 
			this.ResizeExactSize.Location = new System.Drawing.Point(8, 184);
			this.ResizeExactSize.Name = "ResizeExactSize";
			this.ResizeExactSize.Size = new System.Drawing.Size(248, 24);
			this.ResizeExactSize.TabIndex = 3;
			this.ResizeExactSize.Text = "Resize to Exact Size:";
			this.ResizeExactSize.CheckedChanged += new System.EventHandler(this.ResizeExactSize_CheckedChanged);
			// 
			// ResizeAreaPercentage
			// 
			this.ResizeAreaPercentage.Location = new System.Drawing.Point(8, 128);
			this.ResizeAreaPercentage.Name = "ResizeAreaPercentage";
			this.ResizeAreaPercentage.Size = new System.Drawing.Size(248, 24);
			this.ResizeAreaPercentage.TabIndex = 2;
			this.ResizeAreaPercentage.Text = "Scale Area by Percentage:";
			this.ResizeAreaPercentage.CheckedChanged += new System.EventHandler(this.ResizeAreaPercentage_CheckedChanged);
			// 
			// ResizeDimensionPercentage
			// 
			this.ResizeDimensionPercentage.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.ResizeDimensionPercentage.Checked = true;
			this.ResizeDimensionPercentage.Location = new System.Drawing.Point(8, 72);
			this.ResizeDimensionPercentage.Name = "ResizeDimensionPercentage";
			this.ResizeDimensionPercentage.Size = new System.Drawing.Size(248, 24);
			this.ResizeDimensionPercentage.TabIndex = 1;
			this.ResizeDimensionPercentage.TabStop = true;
			this.ResizeDimensionPercentage.Text = "Scale Dimensions by Percentage:";
			this.ResizeDimensionPercentage.CheckedChanged += new System.EventHandler(this.ResizeDimensionPercentage_CheckedChanged);
			// 
			// ResizeMaxDimension
			// 
			this.ResizeMaxDimension.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.ResizeMaxDimension.Location = new System.Drawing.Point(8, 16);
			this.ResizeMaxDimension.Name = "ResizeMaxDimension";
			this.ResizeMaxDimension.Size = new System.Drawing.Size(248, 24);
			this.ResizeMaxDimension.TabIndex = 0;
			this.ResizeMaxDimension.Text = "Scale so Maximum Dimension is:";
			this.ResizeMaxDimension.CheckedChanged += new System.EventHandler(this.ResizeMaxDimension_CheckedChanged);
			// 
			// CurrentFilenameLabel
			// 
			this.CurrentFilenameLabel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.CurrentFilenameLabel.Location = new System.Drawing.Point(8, 352);
			this.CurrentFilenameLabel.Name = "CurrentFilenameLabel";
			this.CurrentFilenameLabel.Size = new System.Drawing.Size(248, 32);
			this.CurrentFilenameLabel.TabIndex = 3;
			this.CurrentFilenameLabel.Text = "Current File: <none>";
			// 
			// ResizeToolTips
			// 
			this.ResizeToolTips.AutoPopDelay = 10000;
			this.ResizeToolTips.InitialDelay = 500;
			this.ResizeToolTips.ReshowDelay = 100;
			this.ResizeToolTips.ShowAlways = true;
			// 
			// EditShowUnprocessedFiles
			// 
			this.EditShowUnprocessedFiles.Index = 2;
			this.EditShowUnprocessedFiles.Text = "Show &Unprocessed Files Only";
			this.EditShowUnprocessedFiles.Click += new System.EventHandler(this.EditShowUnprocessedFiles_Click);
			// 
			// EditShowAllFiles
			// 
			this.EditShowAllFiles.Index = 3;
			this.EditShowAllFiles.Text = "Show All &Files";
			this.EditShowAllFiles.Click += new System.EventHandler(this.EditShowAllFiles_Click);
			// 
			// ImageResizerForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(968, 702);
			this.Controls.Add(this.ResizeSettingsGroupBox);
			this.Controls.Add(this.ResizeProgressBar);
			this.Controls.Add(this.InputFilesListBox);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu;
			this.Name = "ImageResizerForm";
			this.Text = "Image Resizer";
			this.ResizeSettingsGroupBox.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.ExactHeightEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.ExactWidthEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.AreaPercentageEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.DimensionPercentageEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.MaximumDimensionEntry)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new ImageResizerForm());
		}

		// file menu methods and helper functions.
		private void FileSetInputDir_Click(object sender, System.EventArgs e)
		{
			if(folderBrowserDialog.ShowDialog() == DialogResult.OK) 
			{
				// store the input directory name for later.
				InputDir = folderBrowserDialog.SelectedPath;
				// if no output directory is set, set output directory to "inputdirectory\Resized"
				if(OutputDir.Length == 0)
					OutputDir = InputDir + @"\" + "Resized";
				string InputDirText = "Input Directory: " + InputDir,
					   OutputDirText = "Output Directory: " + OutputDir;

				Graphics G = InputDirectoryLabel.CreateGraphics();
				Font F = InputFilesListBox.Font;

				InputDirText  = CleanupDirName(InputDirText,  G, F, InputDirectoryLabel.Size);
				OutputDirText = CleanupDirName(OutputDirText, G, F, OutputDirectoryLabel.Size);

				// update InputDirectoryLabel and OutputDirectoryLabel
				InputDirectoryLabel.Text  = InputDirText;
				OutputDirectoryLabel.Text = OutputDirText;

				// load filenames into the control (false == don't suppress stuff in the output directory);
				LoadFiles(false);

				// enable ResizeButton if there were in fact any image files in the requested directory
				if(InputFilesListBox.Items.Count > 0)
					ResizeButton.Enabled = true;
			}
		}
		// remove unnecessary components from a directory name
		private string CleanupDirName(string name, Graphics G, Font F, SizeF size) 
		{
			Regex Match1  = new Regex(@"\\(Documents and Settings|Desktop|My Documents)\\", RegexOptions.Compiled | RegexOptions.IgnoreCase);

			while( G.MeasureString(name, F, (int)(size.Width)).Height > size.Height && 
				   Match1.IsMatch(name)) 
				name = Match1.Replace(name,"\\...\\",1,0);
			return name;
		}
		/// <summary>
		/// Reads image file names from the input directory and adds them to InputFilesListBox.Items.  If the
		/// suppress parameter is set to true, any file names which exist in both the input and output directories
		/// are suppressed, and not added to InputFilesListBox.Items.
		/// </summary>
		/// <param name="suppress">If true, hides files that already exist in the output directory.</param>
		private void LoadFiles(bool suppress)
		{
			// this regex is used to test whether files are image files.
			Regex imagefileregex = new Regex(
				@"\.(bmp|emf|exif|gif|ico|jpg|jpeg|png|tiff|wmf)$",
				RegexOptions.IgnoreCase | RegexOptions.Compiled);

			// G, ItemWidth, and MaxWidth are used to intelligently set the width of the columns in list box.
			Graphics G = InputFilesListBox.CreateGraphics();
			Font F = InputFilesListBox.Font;
			int ItemWidth = 0, MaxWidth = 0;

			if(InputFilesListBox.Items.Count > 0)  // empty out the list if there's anything in it already
				InputFilesListBox.Items.Clear();
			
			// open the directory
			DirectoryInfo dir = new DirectoryInfo(InputDir);

			//iterate through its files
			foreach (FileInfo f in dir.GetFiles()) 
				if(imagefileregex.IsMatch(f.Name) &&	// if the file is an image file,
				   !(suppress == true &&				// and if we want to do this one,
					 (new FileInfo(OutputDir + @"\" + f.Name)).Exists)) 
					{
						InputFilesListBox.Items.Add(f.Name);
						ItemWidth = (int)(G.MeasureString(f.Name, F).Width);
						if(ItemWidth > MaxWidth) 
							MaxWidth = ItemWidth;
					}
			InputFilesListBox.ColumnWidth = MaxWidth;		// set column width to width of widest item.
			EditSelectHelper(true);							// set everything to selected by default.
		}

		private void FileSetOutputDir_Click(object sender, System.EventArgs e)
		{
			if(folderBrowserDialog.ShowDialog() == DialogResult.OK) 
			{
				// Save the path for later
				OutputDir = folderBrowserDialog.SelectedPath;
				// update OutputDirectoryLabel
				OutputDirectoryLabel.Text = "OutputDirectoryLabel: " + OutputDir;
			}
		}

		private void FileExit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		// edit menu methods and helper function.
		private void EditSelectAll_Click(object sender, System.EventArgs e)
		{
			EditSelectHelper(true);
		}
		private void EditSelectNone_Click(object sender, System.EventArgs e)
		{
			EditSelectHelper(false);
		}
		private void EditShowUnprocessedFiles_Click(object sender, System.EventArgs e)
		{
			InputFilesListBox.Items.Clear(); // empty the list
			LoadFiles(true); // re-load stuff from the input directory (true = suppress stuff that's in the output directory)
			EditSelectHelper(true);
		}
		private void EditShowAllFiles_Click(object sender, System.EventArgs e)
		{
			InputFilesListBox.Items.Clear();
			LoadFiles(false);
			EditSelectHelper(true);
		}
		private void EditSelectHelper(bool NewState) 
		{
			// set checked property of everything in InputFilesListBox to NewState.
			for (int i = 0; i < InputFilesListBox.Items.Count; i++) 
				InputFilesListBox.SetItemCheckState(i, NewState ? CheckState.Checked : CheckState.Unchecked);
		}

		// enable/disable various UI controls depending on what resize mode the user just selected:
		private void ResizeMaxDimension_CheckedChanged(object sender, System.EventArgs e)
		{
			MaximumDimensionEntry.Enabled						= true;
			DimensionPercentageEntry.Enabled					= false;
			AreaPercentageEntry.Enabled							= false;
			ExactWidthEntry.Enabled = ExactHeightEntry.Enabled	= false;
			ResizeMode = "MaxDimension";
		}
		private void ResizeDimensionPercentage_CheckedChanged(object sender, System.EventArgs e)
		{
			MaximumDimensionEntry.Enabled						= false;
			DimensionPercentageEntry.Enabled					= true;
			AreaPercentageEntry.Enabled							= false;
			ExactWidthEntry.Enabled = ExactHeightEntry.Enabled	= false;
			ResizeMode = "DimensionPercentage";
		}
		private void ResizeAreaPercentage_CheckedChanged(object sender, System.EventArgs e)
		{
			MaximumDimensionEntry.Enabled						= false;
			DimensionPercentageEntry.Enabled					= false;
			AreaPercentageEntry.Enabled							= true;
			ExactWidthEntry.Enabled = ExactHeightEntry.Enabled	= false;
			ResizeMode = "AreaPercentage";
		}
		private void ResizeExactSize_CheckedChanged(object sender, System.EventArgs e)
		{
			MaximumDimensionEntry.Enabled						= false;
			DimensionPercentageEntry.Enabled					= false;
			AreaPercentageEntry.Enabled							= false;
			ExactWidthEntry.Enabled = ExactHeightEntry.Enabled	= true;
			ResizeMode = "ExactSize";
		}

		// Ok, here's the heart of the beast.
		private void ResizeButton_Click(object sender, System.EventArgs e)
		{
			DirectoryInfo d = new DirectoryInfo(OutputDir);  // create the output directory, if necessary.
			if(!d.Exists)
				d.Create();

			// set up the progress bar
			ResizeProgressBar.Minimum = 0;
			ResizeProgressBar.Maximum = InputFilesListBox.CheckedIndices.Count;
			ResizeProgressBar.Value	  = 0;
			// disable the list box
			InputFilesListBox.Enabled = false;
			// disable the resize controls and the resize button.
			ResizeSettingsGroupBox.Enabled = false;
			// disable some menu commands
			FileSetInputDir.Enabled  = false;
			FileSetOutputDir.Enabled = false;
			EditSelectAll.Enabled    = false;
			EditSelectNone.Enabled   = false;

			try 
			{
				Thread t = new Thread(new ThreadStart(DoResize));
				t.Name = "ResizeWorkerThread";
				t.IsBackground = true;
				t.Priority = ThreadPriority.Lowest;
				t.Start();
			}
			catch 
			{
				MessageBox.Show("Error: could not start resizing process thread",
								"Resize Error",
								MessageBoxButtons.OK);
				OnResizeDone(this, new EventArgs()); // do all the same stuff as when the thread finishes normally.
			}
		}

		private void DoResize() 
		{
			// iterate over everything in the checked items collection:
			bool success = true;
			for(int i = 0; i < InputFilesListBox.CheckedIndices.Count; i++) 
			{
				string filename   = (InputFilesListBox.Items[InputFilesListBox.CheckedIndices[i]]).ToString();
				string InputPath  = InputDir  + @"\" + filename;	// the true path of the input file
				string OutputPath = OutputDir + @"\" + filename;	// the true path of the output file.
				// pass the current filename back to the UI:
				if(OnFilenameChangeEvent != null)
					OnFilenameChangeEvent(this, new ResizeFilenameArgs(filename));

				// perform the resize-and-save operation
				success &= ResizeFile(InputPath,OutputPath);

				// tell the UI to inform the user that there has been some progress
				if(OnProgressEvent != null)
					OnProgressEvent(this,new EventArgs());
			}

			if(!success)	// warn the user if there were any problems.
				MessageBox.Show("There were some errors during processing.  Some files may not have been created properly",
					"Warning: Resize Errors", MessageBoxButtons.OK);

			// Tell the UI that we're all done
			if(OnResizeDoneEvent != null)
				OnResizeDoneEvent(this,new EventArgs());
		}

		// the event handlers for our three events:
		private void OnFilename(object worker, ResizeFilenameArgs e) 
		{
			if(CurrentFilenameLabel.InvokeRequired) // switch threads if necessary
			{
				this.BeginInvoke(new ResizeFilenameHandler(OnFilename),
					new object[]{worker,e});
			} 
			else 
			{
				CurrentFilenameLabel.Text = "Current File:\n    " + e.filename;
				Bitmap b = new Bitmap(InputDir + @"\" + e.filename);
				double Scale = Math.Min((double)CurrentPictureBox.Width  / (double)b.Width,
										(double)CurrentPictureBox.Height / (double)b.Height);
				Graphics g = CurrentPictureBox.CreateGraphics();
				SolidBrush Brush = new SolidBrush(CurrentPictureBox.BackColor);
				g.FillRectangle(Brush, 0,0, CurrentPictureBox.Width, CurrentPictureBox.Height);
				g.DrawImage(b, 0, 0, (int)((double)b.Width  * Scale), (int)((double)b.Height * Scale) );
			}
		}
		private void OnProgress(object worker, EventArgs e) 
		{
			if(ResizeProgressBar.InvokeRequired)
				this.BeginInvoke(new ResizeProgressHandler(OnProgress),
					new object[]{worker,e});
			else 
			{
				ResizeProgressBar.Increment(1);
				GC.Collect(); // kind of a hack, but it works.
			}
		}

		private void OnResizeDone(object worker, EventArgs e) 
		{
			if(ResizeProgressBar.InvokeRequired)
				this.BeginInvoke(new ResizeDoneHandler(OnResizeDone),
					new object[]{worker,e});
			else 
			{
				// clear the progress bar
				ResizeProgressBar.Value = 0;
				// re-enable the list box
				InputFilesListBox.Enabled = true;
				// re-enable the resize settings controls and the resize button
				ResizeSettingsGroupBox.Enabled = true;
				// re-enable menu commands
				FileSetInputDir.Enabled  = true;
				FileSetOutputDir.Enabled = true;
				EditSelectAll.Enabled    = true;
				EditSelectNone.Enabled   = true;
			}		
		}

		private bool ResizeFile(string InputPath, string OutputPath) 
		{
			// If output file already exists and overwrite mode is off, do nothing and return success.
			if(File.Exists(OutputPath) && OverwriteCheckBox.Checked == false)
				return true;

			try 
			{
				// Load the input bitmap immediately, so we can get it's size.
				Bitmap In = new Bitmap(InputPath);
				int OutWidth, OutHeight;
				// Next figure out the output bitmap size based on what mode we're in.
				switch(ResizeMode) 
				{
					case "MaxDimension":  // scale both dimensions so that the largest dimension is a fixed value.
						double scale = (double)MaximumDimensionEntry.Value / (double)Math.Max(In.Width,In.Height);
						OutWidth  = (int)((double)In.Width * scale);
						OutHeight = (int)((double)In.Height * scale);
						break;
					case "DimensionPercentage":
						OutWidth  = (int)((double)In.Width  * (double)DimensionPercentageEntry.Value/100.0);
						OutHeight = (int)((double)In.Height * (double)DimensionPercentageEntry.Value/100.0);
						break;
					case "AreaPercentage":
						OutWidth  = (int)((double)In.Width  * Math.Sqrt((double)AreaPercentageEntry.Value/100.0));
						OutHeight = (int)((double)In.Height * Math.Sqrt((double)AreaPercentageEntry.Value/100.0));
						break;
					case "ExactSize":
						OutWidth  = (int)(ExactWidthEntry.Value);
						OutHeight = (int)(ExactHeightEntry.Value);
						break;
					default:
						goto case "MaxDimension";
				}
				OutWidth  = Math.Max(OutWidth,1); // ensure at least some minimum size.
				OutHeight = Math.Max(OutHeight,1);

				// create the output bitmap
				Bitmap Out = new Bitmap(OutWidth,OutHeight);

				// Get a graphics object for it, and use GDI to perform the resize operation:
				Graphics g = Graphics.FromImage(Out);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bicubic;
				g.DrawImage(In, 0,0, OutWidth, OutHeight);

				// save results
				Out.Save(OutputPath, GetFormatFromFilename(OutputPath));

				// clean up:
				g.Dispose();
				Out.Dispose();
				return true;
			}
			catch // if anything went wrong, totally bail out.
			{
				return false;
			}
		}

	// Helper function for picking an image format based on file extension.  Why the Bitmap
	// class doesn't include this is totally beyond me...
		private ImageFormat GetFormatFromFilename(string filename) 
		{
			string[] substrings = filename.Split(new char[]{'.'});
			string extension = substrings[substrings.GetUpperBound(0)];
			extension = extension.ToLower();

			switch(extension)
			{
				case "bmp":
					return ImageFormat.Bmp;
				case "emf":
					return ImageFormat.Emf;
				case "exif":
					return ImageFormat.Exif;
				case "gif":
					return ImageFormat.Gif;
				case "ico":
					return ImageFormat.Icon;
				case "jpg":
				case "jpeg":
					return ImageFormat.Jpeg;
				case "png":
					return ImageFormat.Png;
				case "tiff":
					return ImageFormat.Tiff;
				case "wmf":
					return ImageFormat.Wmf;
				default:
					// Show a warning message error message and return the PNG case.
					MessageBox.Show("Unknown image format: " + extension + "\nSaving in PNG format", "Unknown Format",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
					goto case "png";
			}
		}
	}
}
